#!/usr/bin/env python
 
from sklearn.preprocessing import MinMaxScaler as _MinMaxScaler
from base import *
from codec import codecs_manager
import pandas as pd
from command_util import convert_params
 
 
class MinMaxScaler(BaseMixin):
    def __init__(self, options):
        self.handle_options(options)
 
        out_params = convert_params(
            options.get('params', {}),
            strs=['feature_range']
        )
 
 
        if 'feature_range' in out_params.keys():
            # sklearn wants a tuple for minmax_range
            # SPL won't accept a tuple as an option's value
            # but users providing a range such as 1-10, e.g. min=2 max=10 , (2,10)
            try:
                out_params['feature_range'] = tuple(int(i) for i in out_params['feature_range'].split('-'))
                assert len(out_params['feature_range']) == 2
            except:
                raise RuntimeError('Syntax Error: feature_range requires a range, e.g. feature_range=1-5')
 
 
        self.estimator = _MinMaxScaler(**out_params)
        self.columns = None
 
    def preprocess_fit(self, X):
        self.variables = self.match_field_globs(X.columns, self.variables)
        self.drop_unused_fields(X, self.variables)
        self.drop_na_columns(X)
        self.drop_na_rows(X)
        self.warn_on_missing_fields(X, self.variables)
        self.filter_non_numeric(X)
        X = pd.get_dummies(X, prefix_sep='=', sparse=True)
        self.assert_any_fields(X)
        self.assert_any_rows(X)
        columns = self.sort_fields(X)
        return (X, columns)
 
    def fit(self, X):
        X, self.columns = self.preprocess_fit(X)
        self.estimator.fit(X)
 
    def partial_fit(self, X, handle_new_cat):
        X, columns = self.preprocess_fit(X)
        if self.columns is not None:
            self.handle_categorical(X, None, handle_new_cat, self.columns)
            if X.empty:
                return
        else:
            self.columns = columns
        self.estimator.partial_fit(X)
 
    def predict(self, X, options=None, output_name=None):
        if options is not None:
            self.drop_unused_fields(X, self.variables)
            self.drop_na_columns(X)
            self.warn_on_missing_fields(X, self.variables)
            self.filter_non_numeric(X)
            X = pd.get_dummies(X, prefix_sep='=', sparse=True)
            self.drop_unused_fields(X, self.columns)
            self.assert_any_fields(X)
            self.fill_missing_fields(X, self.columns)
            self.sort_fields(X)
            assert set(X.columns) == set(self.columns), 'Internal error: column mismatch'
 
            length = len(X)
            nans = self.drop_na_rows(X)
 
            y_hat = self.estimator.transform(X.values)
 
            # Allocate output DataFrame
            width = y_hat.shape[1]
            columns = ['MM_%s' % col for col in self.columns]
 
            output = pd.DataFrame(np.empty((length, width)), columns = columns)
            output.ix[:,columns] = np.nan
            output.ix[~nans, columns] = y_hat
 
            return output
 
    def summary(self):
        return pd.DataFrame({'fields': self.columns,
                              'min': self.estimator.min_,
                             'scale': self.estimator.scale_})
 
    @staticmethod
    def register_codecs():
        from codec.codecs import SimpleObjectCodec
        codecs_manager.add_codec('algos.MinMaxScaler', 'MinMaxScaler', SimpleObjectCodec)
        codecs_manager.add_codec('sklearn.preprocessing.data', 'MinMaxScaler', SimpleObjectCodec)